@@ -10,6 +10,7 @@ import sys
 import typing
 import warnings
 from concurrent.futures import Future
+from functools import cached_property
 from types import GeneratorType
 from urllib.parse import unquote, urljoin
 
@@ -77,6 +78,16 @@ class _Upgrade(Exception):
         self.session = session
 
 
+class WebSocketDenialResponse(  # type: ignore[misc]
+    httpx.Response,
+    WebSocketDisconnect,
+):
+    """
+    A special case of `WebSocketDisconnect`, raised in the `TestClient` if the
+    `WebSocket` is closed before being accepted with a `send_denial_response()`.
+    """
+
+
 class WebSocketTestSession:
     def __init__(
         self,
@@ -95,7 +106,6 @@ class WebSocketTestSession:
     def __enter__(self) -> WebSocketTestSession:
         self.exit_stack = contextlib.ExitStack()
         self.portal = self.exit_stack.enter_context(self.portal_factory())
-        self.should_close = anyio.Event()
 
         try:
             _: Future[None] = self.portal.start_task_soon(self._run)
@@ -109,6 +119,10 @@ class WebSocketTestSession:
         self.extra_headers = message.get("headers", None)
         return self
 
+    @cached_property
+    def should_close(self) -> anyio.Event:
+        return anyio.Event()
+
     async def _notify_close(self) -> None:
         self.should_close.set()
 
@@ -155,7 +169,22 @@ class WebSocketTestSession:
     def _raise_on_close(self, message: Message) -> None:
         if message["type"] == "websocket.close":
             raise WebSocketDisconnect(
-                message.get("code", 1000), message.get("reason", "")
+                code=message.get("code", 1000), reason=message.get("reason", "")
+            )
+        elif message["type"] == "websocket.http.response.start":
+            status_code: int = message["status"]
+            headers: list[tuple[bytes, bytes]] = message["headers"]
+            body: list[bytes] = []
+            while True:
+                message = self.receive()
+                assert message["type"] == "websocket.http.response.body"
+                body.append(message["body"])
+                if not message.get("more_body", False):
+                    break
+            raise WebSocketDenialResponse(
+                status_code=status_code,
+                headers=headers,
+                content=b"".join(body),
             )
 
     def send(self, message: Message) -> None:
@@ -273,6 +302,7 @@ class _TestClientTransport(httpx.BaseTransport):
                 "server": [host, port],
                 "subprotocols": subprotocols,
                 "state": self.app_state.copy(),
+                "extensions": {"websocket.http.response": {}},
             }
             session = WebSocketTestSession(self.app, scope, self.portal_factory)
             raise _Upgrade(session)
